module display(clk,
               en, wr,
               rdwr_row, rdwr_col,
               wr_data, rd_data,
               hsync, vsync,
               r, g, b);
    input clk;
    input en;
    input wr;
    input [4:0] rdwr_row;
    input [6:0] rdwr_col;
    input [7:0] wr_data;
    output [7:0] rd_data;
    output hsync;
    output vsync;
    output r;
    output g;
    output b;

  wire pixclk;

  wire [4:0] txtrow_timing;
  wire [6:0] txtcol_timing;
  wire [3:0] chrrow_timing;
  wire [2:0] chrcol_timing;
  wire blank_timing;
  wire hsync_timing;
  wire vsync_timing;

  wire [7:0] chrcode_dspmem;
  wire [3:0] chrrow_dspmem;
  wire [2:0] chrcol_dspmem;
  wire blank_dspmem;
  wire hsync_dspmem;
  wire vsync_dspmem;

  wire pixel_chrgen;
  wire blank_chrgen;
  wire hsync_chrgen;
  wire vsync_chrgen;

  timing timing1(clk,
                 pixclk,
                 txtrow_timing, txtcol_timing,
                 chrrow_timing, chrcol_timing,
                 blank_timing, hsync_timing, vsync_timing);

  dspmem dspmem1(rdwr_row, rdwr_col, wr_data, rd_data, en, wr,
                 clk, pixclk,
                 txtrow_timing, txtcol_timing, chrcode_dspmem,
                 chrrow_timing, chrcol_timing,
                 blank_timing, hsync_timing, vsync_timing,
                 chrrow_dspmem, chrcol_dspmem,
                 blank_dspmem, hsync_dspmem, vsync_dspmem);

  chrgen chrgen1(clk, pixclk,
                 chrcode_dspmem, chrrow_dspmem, chrcol_dspmem,
                 pixel_chrgen,
                 blank_dspmem, hsync_dspmem, vsync_dspmem,
                 blank_chrgen, hsync_chrgen, vsync_chrgen);

  pixel pixel1(clk, pixclk,
               pixel_chrgen,
               blank_chrgen, hsync_chrgen, vsync_chrgen,
               r, g, b, hsync, vsync);

endmodule
