module dspmem(rdwr_row, rdwr_col, wr_data, rd_data, en, wr,
              clk, pixclk,
              txtrow, txtcol, chrcode,
              chrrow_in, chrcol_in, blank_in, hsync_in, vsync_in,
              chrrow_out, chrcol_out, blank_out, hsync_out, vsync_out);
    input [4:0] rdwr_row;
    input [6:0] rdwr_col;
    input [7:0] wr_data;
    output [7:0] rd_data;
    input en;
    input wr;
    input clk;
    input pixclk;
    input [4:0] txtrow;
    input [6:0] txtcol;
    output [7:0] chrcode;
    input [3:0] chrrow_in;
    input [2:0] chrcol_in;
    input blank_in;
    input hsync_in;
    input vsync_in;
    output reg [3:0] chrrow_out;
    output reg [2:0] chrcol_out;
    output reg blank_out;
    output reg hsync_out;
    output reg vsync_out;

  wire [11:0] addr_wr;
  wire [3:0] din_wr_hi;
  wire [3:0] din_wr_lo;
  wire [3:0] dout_wr_hi;
  wire [3:0] dout_wr_lo;

  wire [11:0] addr_rd;
  wire [3:0] din_rd_hi;
  wire [3:0] din_rd_lo;
  wire [3:0] dout_rd_hi;
  wire [3:0] dout_rd_lo;

  assign addr_wr[11:7] = rdwr_row[4:0];
  assign addr_wr[6:0] = rdwr_col[6:0];
  assign din_wr_hi = wr_data[7:4];
  assign din_wr_lo = wr_data[3:0];
  assign rd_data[7:4] = dout_wr_hi;
  assign rd_data[3:0] = dout_wr_lo;

  assign addr_rd[11:7] = txtrow[4:0];
  assign addr_rd[6:0] = txtcol[6:0];
  assign din_rd_hi = 4'b0000;
  assign din_rd_lo = 4'b0000;
  assign chrcode[7:4] = dout_rd_hi;
  assign chrcode[3:0] = dout_rd_lo;

  // RAMB16_S4_S4: Spartan-3 4k x 4 Dual-Port RAM

  RAMB16_S4_S4 display_mem_hi (
    .DOA(dout_wr_hi),  // Port A 4-bit Data Output
    .DOB(dout_rd_hi),  // Port B 4-bit Data Output
    .ADDRA(addr_wr),   // Port A 12-bit Address Input
    .ADDRB(addr_rd),   // Port B 12-bit Address Input
    .CLKA(clk),        // Port A Clock
    .CLKB(clk),        // Port B Clock
    .DIA(din_wr_hi),   // Port A 4-bit Data Input
    .DIB(din_rd_hi),   // Port B 4-bit Data Input
    .ENA(en),          // Port A RAM Enable Input
    .ENB(pixclk),      // Port B RAM Enable Input
    .SSRA(1'b0),       // Port A Synchronous Set/Reset Input
    .SSRB(1'b0),       // Port B Synchronous Set/Reset Input
    .WEA(wr),          // Port A Write Enable Input
    .WEB(1'b0)         // Port B Write Enable Input
  );

  defparam display_mem_hi.INIT_A = 18'h0; // Value of output RAM registers on Port A at startup
  defparam display_mem_hi.INIT_B = 18'h0; // Value of output RAM registers on Port B at startup
  defparam display_mem_hi.SRVAL_A = 18'h0; // Port A ouput value upon SSR assertion
  defparam display_mem_hi.SRVAL_B = 18'h0; // Port B ouput value upon SSR assertion
  defparam display_mem_hi.WRITE_MODE_A = "WRITE_FIRST"; // WRITE_FIRST, READ_FIRST or NO_CHANGE
  defparam display_mem_hi.WRITE_MODE_B = "WRITE_FIRST"; // WRITE_FIRST, READ_FIRST or NO_CHANGE

  `include "dspmemhi.init"

  // RAMB16_S4_S4: Spartan-3 4k x 4 Dual-Port RAM

  RAMB16_S4_S4 display_mem_lo (
    .DOA(dout_wr_lo),  // Port A 4-bit Data Output
    .DOB(dout_rd_lo),  // Port B 4-bit Data Output
    .ADDRA(addr_wr),   // Port A 12-bit Address Input
    .ADDRB(addr_rd),   // Port B 12-bit Address Input
    .CLKA(clk),        // Port A Clock
    .CLKB(clk),        // Port B Clock
    .DIA(din_wr_lo),   // Port A 4-bit Data Input
    .DIB(din_rd_lo),   // Port B 4-bit Data Input
    .ENA(en),          // Port A RAM Enable Input
    .ENB(pixclk),      // Port B RAM Enable Input
    .SSRA(1'b0),       // Port A Synchronous Set/Reset Input
    .SSRB(1'b0),       // Port B Synchronous Set/Reset Input
    .WEA(wr),          // Port A Write Enable Input
    .WEB(1'b0)         // Port B Write Enable Input
  );

  defparam display_mem_lo.INIT_A = 18'h0; // Value of output RAM registers on Port A at startup
  defparam display_mem_lo.INIT_B = 18'h0; // Value of output RAM registers on Port B at startup
  defparam display_mem_lo.SRVAL_A = 18'h0; // Port A ouput value upon SSR assertion
  defparam display_mem_lo.SRVAL_B = 18'h0; // Port B ouput value upon SSR assertion
  defparam display_mem_lo.WRITE_MODE_A = "WRITE_FIRST"; // WRITE_FIRST, READ_FIRST or NO_CHANGE
  defparam display_mem_lo.WRITE_MODE_B = "WRITE_FIRST"; // WRITE_FIRST, READ_FIRST or NO_CHANGE

  `include "dspmemlo.init"

  always @(posedge clk) begin
    if (pixclk == 1) begin
      chrrow_out <= chrrow_in;
      chrcol_out <= chrcol_in;
      blank_out <= blank_in;
      hsync_out <= hsync_in;
      vsync_out <= vsync_in;
    end
  end

endmodule
